use std::collections::HashMap;

#[derive(Default)]
pub(crate) struct State {
    nodes: HashMap<u32, Node>,
    links: HashMap<u32, Link>,
}

#[derive(Debug)]
struct Node {
    id: u32,
    name: Option<String>,
}

#[derive(Debug)]
struct Link {
    id: u32,
    output: u32,
    input: u32,
}

impl State {
    pub(crate) fn add_node<N: Into<String>>(&mut self, id: u32, name: Option<N>) {
        let name = name.map(Into::into);
        println!("Node Add {} {:?}", id, name);
        let node = Node { id, name };
        self.nodes.insert(id, node);
    }

    pub(crate) fn add_link(&mut self, id: u32, input: u32, output: u32) {
        println!("Link Add {:?} {:?} {:?}", id, input, output);
        let link = Link { id, input, output };
        self.links.insert(id, link);
    }

    pub(crate) fn remove_id(&mut self, id: u32) {
        if let Some(node) = self.nodes.remove(&id) {
            println!("Node Remove {} {:?}", id, node);
        } else if let Some(link) = self.links.remove(&id) {
            println!("Link Remove {} {:?}", id, link);
        } else {
            eprintln!("Unrecognized remove: {}", id);
        }
    }
}
